@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # synadm
-# Copyright (C) 2020-2022 Johannes Tiefenbacher
+# Copyright (C) 2020-2023 Johannes Tiefenbacher
 #
 # synadm is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,6 +33,15 @@ import re
 from synadm import api
 
 
+output_format_help = """The 'human' mode gives a tabular or list view depending
+on the fetched data, but often needs a lot of horizontal space to display
+correctly. 'json' returns formatted json. 'minified' is minified json, suitable
+for scripting purposes. 'pprint' shows a formatted output with the help of
+Python's built-in pprint module. 'yaml' is a compromise between human- and
+machine-readable output, it doesn't need as much terminal width as 'human' does
+and is the default on fresh installations."""
+
+
 def humanize(data):
     """ Try to display data in a human-readable form:
     - Lists of dicts are displayed as tables.
@@ -57,7 +66,8 @@ class APIHelper:
 
     FORMATTERS = {
         "pprint": pprint.pformat,
-        "json": json.dumps,
+        "json": lambda data: json.dumps(data, indent=4),
+        "minified": lambda data: json.dumps(data, separators=(",", ":")),
         "yaml": yaml.dump,
         "human": humanize
     }
@@ -70,13 +80,14 @@ class APIHelper:
         "matrix_path": "/_matrix",
         "timeout": 30,
         "server_discovery": "well-known",
-        "homeserver": "auto-retrieval"
+        "homeserver": "auto-retrieval",
+        "ssl_verify": True
     }
 
-    def __init__(self, config_path, verbose, batch, output_format_cli):
+    def __init__(self, config_path, verbose, no_confirm, output_format_cli):
         self.config = APIHelper.CONFIG.copy()
         self.config_path = os.path.expanduser(config_path)
-        self.batch = batch
+        self.no_confirm = no_confirm
         self.api = None
         self.init_logger(verbose)
         self.requests_debug = False
@@ -129,8 +140,13 @@ class APIHelper:
         except Exception as error:
             self.log.error("%s while reading configuration file", error)
         for key, value in self.config.items():
-            if not value:
-                self.log.error("Config entry missing: %s", key)
+
+            if key == "ssl_verify" and type(value) != bool:
+                self.log.error("Config value error: %s, %s must be boolean",
+                               key, value)
+
+            if not value and type(value) != bool:
+                self.log.error("Config entry missing: %s, %s", key, value)
                 return False
             else:
                 if key == "token":
@@ -145,17 +161,20 @@ class APIHelper:
             self.log,
             self.config["user"], self.config["token"],
             self.config["base_url"], self.config["admin_path"],
-            self.config["timeout"], self.requests_debug
+            self.config["timeout"], self.requests_debug,
+            self.config["ssl_verify"]
         )
         self.matrix_api = api.Matrix(
             self.log,
             self.config["user"], self.config["token"],
             self.config["base_url"], self.config["matrix_path"],
-            self.config["timeout"], self.requests_debug
+            self.config["timeout"], self.requests_debug,
+            self.config["ssl_verify"]
         )
         self.misc_request = api.MiscRequest(
             self.log,
             self.config["timeout"], self.requests_debug,
+            self.config["ssl_verify"]
         )
         return True
 
@@ -201,7 +220,7 @@ class APIHelper:
             string: hostname, FQDN or DOMAIN; or None on errors.
         """
         uri = uri if uri else self.config["base_url"]
-        echo = self.log.info if self.batch else click.echo
+        echo = self.log.info if self.no_confirm else click.echo
         if self.config["homeserver"] != "auto-retrieval":
             return self.config["homeserver"]
 
@@ -285,9 +304,9 @@ class APIHelper:
     "--verbose", "-v", count=True, default=False,
     help="Enable INFO (-v) or DEBUG (-vv) logging on console.")
 @click.option(
-    "--batch", "--yes", "--non-interactive", "--scripting",
+    "--no-confirm", "--batch", "--yes", "--non-interactive", "--scripting",
     default=False, is_flag=True,
-    help="""Enable batch processing mode. Use with caution! This will:
+    help="""Enable non-interactive mode. Use with caution! This will:
 
     \b
         - Disable all interactive prompts.
@@ -296,22 +315,22 @@ class APIHelper:
     """)
 @click.option(
     "--output", "-o", default="",
-    type=click.Choice(["yaml", "json", "human", "pprint",
-                       "y", "j", "h", "p", ""]),
+    type=click.Choice(["yaml", "json", "minified", "human", "pprint",
+                       "y", "j", "m", "h", "p", ""]),
     show_choices=True,
-    help="""Override default output format.""")
+    help=f"Override default output format. {output_format_help}")
 @click.option(
     "--config-file", "-c", type=click.Path(),
     default="~/.config/synadm.yaml",
     help="Configuration file path.", show_default=True)
 @click.pass_context
-def root(ctx, verbose, batch, output, config_file):
+def root(ctx, verbose, no_confirm, output, config_file):
     """ the Matrix-Synapse admin CLI
     """
-    ctx.obj = APIHelper(config_file, verbose, batch, output)
+    ctx.obj = APIHelper(config_file, verbose, no_confirm, output)
     helper_loaded = ctx.obj.load()
     if ctx.invoked_subcommand != "config" and not helper_loaded:
-        if batch:
+        if no_confirm:
             click.echo("Please setup synadm: " + sys.argv[0] + " config.")
             raise SystemExit(2)
         else:
@@ -344,14 +363,11 @@ def root(ctx, verbose, batch, output, config_file):
     help="""The time in seconds synadm should wait for responses from admin
     API's or Matrix API's. The default is 7 seconds. """)
 @click.option(
-    "--output", "-o", type=click.Choice(["yaml", "json", "human", "pprint"]),
-    help="""How synadm displays data by default. 'human' gives a tabular or
-    list view depending on the fetched data. This mode needs your terminal to
-    be quite wide! 'json' displays exactly as the API responded. 'pprint' shows
-    nicely formatted json. 'yaml' is the currently recommended output format.
-    It doesn't need as much terminal width as 'human' does. Note that the
-    default output format can always be overridden by using global switch -o
-    (eg 'synadm -o pprint user list').""")
+    "--output", "-o", type=click.Choice([
+        "yaml", "json", "minified", "human", "pprint"]),
+    help=f"""How synadm displays data by default. {output_format_help} The
+    default output format can always be overridden by using the global
+    --output/-o switch (eg 'synadm -o pprint user list').""")
 @click.option(
     "--server-discovery", "-d", type=click.Choice(["well-known", "dns"]),
     help="""The method used for discovery of "the own homeserver name". Since
@@ -376,12 +392,18 @@ def root(ctx, verbose, batch, output, config_file):
     default value 'auto-retrieval' will try to discover the name using the
     method set by --server-discovery."""
 )
+@click.option(
+    "--ssl-verify", "-i",  is_flag=True, show_default=True,
+    help="""Whether or not SSL certificates should be verified. Set to False
+    to allow self-signed certifcates."""
+)
 @click.pass_obj
 def config_cmd(helper, user_, token, base_url, admin_path, matrix_path,
-               output, timeout, server_discovery, homeserver):
-    """ Modify synadm's configuration. Configuration details are generally
-    always asked interactively. Command line options override the suggested
-    defaults in the prompts.
+               output, timeout, server_discovery, homeserver, ssl_verify):
+    """ Modify synadm's configuration.
+
+    Configuration details are generally always asked interactively. Command
+    line options override the suggested defaults in the prompts.
     """
     def get_redacted_token_prompt(cli_token):
         redacted = "NOT SET"  # Show as empty: [].
@@ -393,11 +415,12 @@ def config_cmd(helper, user_, token, base_url, admin_path, matrix_path,
                 redacted = "REDACTED"  # Token found in config, show [REDACTED]
         return f"Synapse admin user token [{redacted}]"
 
-    if helper.batch:
+    if helper.no_confirm:
         if not all([user, token, base_url, admin_path, matrix_path,
-                    output, timeout, server_discovery, homeserver]):
+                    output, timeout, server_discovery, homeserver,
+                    ssl_verify]):
             click.echo(
-                "Missing config options for batch configuration!"
+                "Missing config options for non-interactive configuration!"
             )
             raise SystemExit(3)
         else:
@@ -411,7 +434,8 @@ def config_cmd(helper, user_, token, base_url, admin_path, matrix_path,
                 "format": output,
                 "timeout": timeout,
                 "server_discovery": server_discovery,
-                "homeserver": homeserver
+                "homeserver": homeserver,
+                "ssl_verify": ssl_verify
             }):
                 raise SystemExit(0)
             else:
@@ -441,7 +465,8 @@ def config_cmd(helper, user_, token, base_url, admin_path, matrix_path,
         "format": click.prompt(
             "Default output format",
             default=output if output else helper.config.get("format", output),
-            type=click.Choice(["yaml", "json", "human", "pprint"])),
+            type=click.Choice([
+                "yaml", "json", "minified", "human", "pprint"])),
         "timeout": click.prompt(
             "Default http timeout",
             default=timeout if timeout else helper.config.get(
@@ -450,6 +475,11 @@ def config_cmd(helper, user_, token, base_url, admin_path, matrix_path,
             "Homeserver name (auto-retrieval or matrix.DOMAIN)",
             default=homeserver if homeserver else helper.config.get(
                 "homeserver", homeserver)),
+        "ssl_verify": click.prompt(
+            "Verify certificate",
+            type=bool,
+            default=ssl_verify if ssl_verify else helper.config.get(
+                "ssl_verify", ssl_verify)),
         "server_discovery": click.prompt(
             "Server discovery mode (used with homeserver name auto-retrieval)",
             default=server_discovery if server_discovery else helper.config.get(  # noqa: E501
