@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # synadm
-# Copyright (C) 2020-2022 Johannes Tiefenbacher
+# Copyright (C) 2020-2023 Johannes Tiefenbacher
 #
 # synadm is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@ from synadm import cli
 
 @cli.root.group()
 def media():
-    """ Manage local and remote media
+    """ Manage local and remote media.
     """
 
 
@@ -75,7 +75,7 @@ def media():
 @click.pass_context
 def media_list_cmd(ctx, helper, room_id, user_id, from_, limit, sort, reverse,
                    datetime):
-    """ List local media by room or user
+    """ List local media by room or user.
     """
     if room_id:
         media_list = helper.api.room_media_list(room_id)
@@ -118,7 +118,7 @@ def media_list_cmd(ctx, helper, room_id, user_id, from_, limit, sort, reverse,
     """)
 @click.pass_obj
 def media_quarantine_cmd(helper, server_name, media_id, user_id, room_id):
-    """ Quarantine media in rooms, by users or by media ID
+    """ Quarantine media in rooms, by users or by media ID.
     """
     if media_id and not server_name:
         # We assume it is local media and fetch our own server name.
@@ -142,11 +142,48 @@ def media_quarantine_cmd(helper, server_name, media_id, user_id, room_id):
     helper.output(media_quarantined)
 
 
+@media.command(name="unquarantine")
+@optgroup.group(
+    "unquarantine media by",
+    cls=RequiredAnyOptionGroup,
+    help="")
+@optgroup.option(
+    "--media-id", "-i", type=str,
+    help="""The media with this specific media ID will be removed from
+    quarantine.
+    """)
+@click.option(
+    "--server-name", "-s", type=str,
+    help="""The server name of the media, mandatory when --media-id is used and
+    _remote_ media should be processed. For locally stored media this option
+    can be omitted.
+    """)
+@click.pass_obj
+def media_unquarantine_cmd(helper, server_name, media_id):
+    """ Remove media from quarantine.
+    """
+    if media_id and not server_name:
+        # We assume it is local media and fetch our own server name.
+        fetched_name = helper.retrieve_homeserver_name(
+            helper.config["base_url"])
+        unquarantinend = helper.api.media_unquarantine(fetched_name, media_id)
+    elif server_name and not media_id:
+        click.echo("Media ID missing.")
+        unquarantinend = None
+    elif media_id and server_name:
+        unquarantinend = helper.api.media_unquarantine(server_name, media_id)
+
+    if unquarantinend is None:
+        click.echo("Media could not be removed from quarantine.")
+        raise SystemExit(1)
+    helper.output(unquarantinend)
+
+
 @media.command(name="protect")
 @click.argument("media_id", type=str)
 @click.pass_obj
 def media_protect_cmd(helper, media_id):
-    """ Protect specific media from being quarantined
+    """ Protect specific media from being quarantined.
     """
     media_protected = helper.api.media_protect(media_id)
     if media_protected is None:
