@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # synadm
-# Copyright (C) 2020-2022 Johannes Tiefenbacher
+# Copyright (C) 2020-2023 Johannes Tiefenbacher
 #
 # synadm is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ from synadm import cli
 
 @cli.root.group()
 def room():
-    """ Manipulate rooms and room membership
+    """ Manipulate rooms and room membership.
 
     The syntax of room IDs in synadm is as the Matrix spec defines:
     https://spec.matrix.org/latest/#room-structure.
@@ -39,7 +39,7 @@ def room():
 @click.argument("user_id", type=str)
 @click.pass_obj
 def join(helper, room_id_or_alias, user_id):
-    """ Join a room
+    """ Join a room.
     """
     mxid = helper.generate_mxid(user_id)
     out = helper.api.room_join(room_id_or_alias, mxid)
@@ -55,7 +55,7 @@ def join(helper, room_id_or_alias, user_id):
     """)
 @click.pass_obj
 def resolve(helper, room_id_or_alias, reverse):
-    """ Lookup room ID from alias or vice versa
+    """ Lookup room ID from alias or vice versa.
     """
     if reverse:
         out = helper.matrix_api.room_get_aliases(room_id_or_alias)
@@ -112,7 +112,7 @@ def list_room_cmd(helper, from_, limit, name, sort, reverse):
 @click.argument("room_id", type=str)
 @click.pass_obj
 def details(helper, room_id):
-    """ Get room details
+    """ Get room details.
     """
     room_details = helper.api.room_details(room_id)
     if room_details is None:
@@ -201,7 +201,7 @@ def power_levels(helper, room_id, all_details, from_, limit, name, sort,
 @click.argument("room_id", type=str)
 @click.pass_obj
 def members(helper, room_id):
-    """ List current room members
+    """ List current room members.
     """
     room_members = helper.api.room_members(room_id)
     if room_members is None:
@@ -249,12 +249,21 @@ def members(helper, room_id):
 @click.pass_context
 def delete(ctx, helper, room_id, new_room_user_id, room_name, message, block,
            no_purge):
-    """ Delete and possibly purge a room
+    """ Delete and possibly purge a room.
     """
-    ctx.invoke(details, room_id=room_id)
+    room_details = helper.api.room_details(room_id)
+    if "errcode" in room_details.keys():
+        if room_details["errcode"] == "M_NOT_FOUND":
+            click.echo("Room not found.")
+            raise SystemExit(1)
+        else:
+            click.echo("Unrecognized error")
+            helper.output(room_details)
+            raise SystemExit(1)
+    helper.output(room_details)
     ctx.invoke(members, room_id=room_id)
     sure = (
-        helper.batch or
+        helper.no_confirm or
         click.prompt("Are you sure you want to delete this room? (y/N)",
                      type=bool, default=False, show_default=False)
     )
@@ -315,3 +324,27 @@ def make_admin(helper, room_id, user_id):
     mxid = helper.generate_mxid(user_id)
     out = helper.api.room_make_admin(room_id, mxid)
     helper.output(out)
+
+
+@room.command()
+@click.argument("room_id", type=str)
+@click.option(
+    "--block/--unblock", "-b/-u", type=bool, default=True, show_default=True,
+    help="Specifies whether to block or unblock a room."
+)
+@click.pass_obj
+def block(helper, room_id, block):
+    """ Block or unblock a room.
+    """
+    out = helper.api.block_room(room_id, block)
+    helper.output(out)
+
+
+@room.command()
+@click.argument("room_id", type=str)
+@click.pass_obj
+def block_status(helper, room_id):
+    """ Get if a room is blocked, and who blocked it.
+    """
+    out = helper.api.room_block_status(room_id)
+    helper.output(out)
