@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # synadm
-# Copyright (C) 2020-2022 Johannes Tiefenbacher
+# Copyright (C) 2020-2023 Johannes Tiefenbacher
 #
 # synadm is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 """ User-related CLI commands
 """
 
+import re
 import click
 from click_option_group import optgroup, MutuallyExclusiveOptionGroup
 from click_option_group import RequiredAnyOptionGroup
@@ -34,7 +35,7 @@ def get_function(function_name):
 
 @cli.root.group()
 def user():
-    """ List, add, modify, deactivate/erase users, reset passwords
+    """ List, add, modify, deactivate/erase users, reset passwords.
     """
 
 
@@ -67,7 +68,7 @@ def user():
     (@user:server) that contain this value""")
 @click.pass_obj
 def list_user_cmd(helper, from_, limit, guests, deactivated, name, user_id):
-    """ List and search for users
+    """ List users, search for users.
     """
     mxid = helper.generate_mxid(user_id)
     users = helper.api.user_list(from_, limit, guests, deactivated, name,
@@ -116,7 +117,7 @@ def deactivate(ctx, helper, user_id, gdpr_erase):
     m_erase_or_deact = "gdpr-erase" if gdpr_erase else "deactivate"
     m_erase_or_deact_p = "gdpr-erased" if gdpr_erase else "deactivated"
     sure = (
-        helper.batch or
+        helper.no_confirm or
         click.prompt("Are you sure you want to {} this user? (y/N)"
                      .format(m_erase_or_deact),
                      type=bool, default=False, show_default=False)
@@ -138,6 +139,62 @@ def deactivate(ctx, helper, user_id, gdpr_erase):
         click.echo("Abort.")
 
 
+@user.command()
+@click.argument("regex", type=str)
+@click.option(
+    "--gdpr-erase", "-e", is_flag=True, default=False,
+    help="""Marks the users as GDPR-erased. This means messages sent by the
+    users will still be visible by anyone that was in the room when these
+    messages were sent, but hidden from other users joining the room
+    afterwards.""", show_default=True
+)
+@click.option(
+    "--dry-run", "-n", is_flag=True, default=False,
+    help="""Do everything except deactivating users."""
+)
+@click.option(
+    "--batch-size", "--paginate", "-p", type=int, default=500,
+    show_default=True,
+    help="""How many users should be requested from the API one at a time.
+    This option has no effect on how many users will be deactivated.
+
+    Increasing this is not necessary in most cases but useful if you have a
+    lot of accounts on your homeserver."""
+)
+@click.pass_obj
+@click.pass_context
+def deactivate_regex(ctx, helper, regex, gdpr_erase, dry_run, batch_size):
+    """ Deactivate or GDPR-erase accounts based on regex.
+
+    Does everything normal deactivation does, just for multiple users matching
+    the given regex.
+
+    The regex argument takes a string and uses Python's re.match (matches
+    regex starting from first character). A regex is expected to match a
+    full Matrix ID, or partially at least from the first character.
+
+    --dry-run can be used to see which accounts will be deactivated. This
+    can be useful for reviewing the accounts that will be deactivated.
+
+    Additionally, the --batch argument (given before the subcommands) will
+    not prompt for if you want to deactivate a user (very useful for many
+    users)."""
+    helper.log.debug(f"Regex: {regex}")
+    # if below fails, turn on debug mode to get the actual given regex.
+    pattern = re.compile(regex)
+    for list_user_response in helper.api.user_list_paginate(batch_size,
+                                                            True, False, "",
+                                                            ""):
+        for user in list_user_response["users"]:
+            if pattern.match(user["name"]):
+                if dry_run:
+                    click.echo(f"Would deactivate: {user['name']}")
+                    continue
+                else:
+                    ctx.invoke(deactivate, user_id=user["name"],
+                               gdpr_erase=gdpr_erase)
+
+
 @user.command(name="prune-devices")
 @click.argument("user_id", type=str)
 @click.option(
@@ -358,10 +415,18 @@ class UserModifyOptionGroup(RequiredAnyOptionGroup):
     removes their active access tokens, resets their password, kicks them out
     of all rooms and deletes third-party identifiers (to prevent the user
     requesting a password reset). See also "user deactivate" command.""")
+@optgroup.option(
+    "--user-type", type=str, default=None, show_default=True,
+    help="""Change the type of the user. Currently understood by the Admin API
+    are 'bot' and 'support'. Use 'regular' to create a regular Matrix user
+    (which effectively sets the user-type to 'null'). If the --user-type option
+    is omitted when modifying an existing user, the user-type will not be
+    manipulated. If the --user-type option is omitted when creating a new user,
+    a regular user will be created.""")
 @click.pass_obj
 @click.pass_context
 def modify(ctx, helper, user_id, password, password_prompt, display_name,
-           threepid, avatar_url, admin, deactivation):
+           threepid, avatar_url, admin, deactivation, user_type):
     """ Create or modify a local user. Provide matrix user ID (@user:server)
     as argument.
     """
@@ -390,12 +455,15 @@ def modify(ctx, helper, user_id, password, password_prompt, display_name,
                             f"{t_key} is probably not a supported medium "
                             "type. Threepid medium types according to the "
                             "current matrix spec are: email, msisdn.")
+        elif key == "user_type" and value == 'regular':
+            click.echo("user_type: null")
         elif value not in [None, {}, []]:  # only show non-empty (aka changed)
             click.echo(f"{key}: {value}")
 
     if password_prompt:
-        if helper.batch:
-            click.echo("Password prompt not available in batch mode. Use -P.")
+        if helper.no_confirm:
+            click.echo("Password prompt not available in non-interactive "
+                       "mode. Use -P.")
         else:
             password = click.prompt("Password", hide_input=True,
                                     confirmation_prompt=True)
@@ -404,14 +472,15 @@ def modify(ctx, helper, user_id, password, password_prompt, display_name,
     else:
         password = None
     sure = (
-        helper.batch or
+        helper.no_confirm or
         click.prompt("Are you sure you want to modify user? (y/N)",
                      type=bool, default=False, show_default=False)
     )
     if sure:
         modified = helper.api.user_modify(
             mxid, password, display_name, threepid,
-            avatar_url, admin, deactivation)
+            avatar_url, admin, deactivation,
+            'null' if user_type == 'regular' else user_type)
         if modified is None:
             click.echo("User could not be modified.")
             raise SystemExit(1)
@@ -551,7 +620,7 @@ def user_login_cmd(helper, user_id, expire_days, expire, expire_ts,
         user_login = helper.api.user_login(mxid, expire_days, expire,
                                            expire_ts)
 
-    if helper.batch:
+    if helper.no_confirm:
         if user_login is None:
             raise SystemExit(1)
         helper.output(user_login)
